'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var toTitleCase = require('to-title-case');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var toTitleCase__default = /*#__PURE__*/_interopDefaultLegacy(toTitleCase);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

function generateTitle(name, title) {
    if (title) {
        return title;
    }
    else if (!name || title === '') {
        return undefined;
    }
    return toTitleCase__default['default'](name);
}

var subgenerate = function (generator) { return (generator.generate ? generator.generate() : generator); };
var subgenerateMany = function (generators) {
    return generators.map(subgenerate);
};

var BaseGenerator = /** @class */ (function () {
    function BaseGenerator(name, title) {
        this._name = name;
        this._title = title;
    }
    BaseGenerator.prototype.title = function (title) {
        this._title = title;
        return this;
    };
    BaseGenerator.prototype.name = function (name) {
        this._name = name;
        return this;
    };
    return BaseGenerator;
}());
var StandardGenerator = /** @class */ (function (_super) {
    __extends(StandardGenerator, _super);
    function StandardGenerator(type, name, title) {
        var _this = _super.call(this, name, title) || this;
        _this._readOnly = false;
        _this._hidden = false;
        _this._options = {};
        _this._type = type;
        return _this;
    }
    StandardGenerator.prototype.description = function (description) {
        this._description = description;
        return this;
    };
    StandardGenerator.prototype.readOnly = function (readOnly) {
        if (readOnly === void 0) { readOnly = true; }
        this._readOnly = readOnly;
        return this;
    };
    StandardGenerator.prototype.hidden = function (hidden) {
        if (hidden === void 0) { hidden = true; }
        this._hidden = hidden;
        return this;
    };
    StandardGenerator.prototype.validation = function (fn) {
        this._validation = fn;
        return this;
    };
    StandardGenerator.prototype.preview = function (preview) {
        this._preview = preview;
        return this;
    };
    StandardGenerator.prototype.fieldset = function (fieldset) {
        this._fieldset = fieldset;
        return this;
    };
    StandardGenerator.prototype.options = function (options) {
        this._options = options;
        return this;
    };
    StandardGenerator.prototype.option = function (property, value) {
        this._options[property] = value;
        return this;
    };
    // @ts-ignore
    StandardGenerator.prototype.extendProperties = function (field) { };
    StandardGenerator.prototype.generate = function () {
        if (!this._type)
            throw Error('Type is required');
        var field = {
            type: this._type,
        };
        if (this._name) {
            field.name = this._name;
        }
        var title = generateTitle(this._name, this._title);
        if (title) {
            field.title = title;
        }
        if (this._description) {
            field.description = this._description;
        }
        if (this._readOnly) {
            field.readOnly = this._readOnly;
        }
        if (this._hidden) {
            field.hidden = this._hidden;
        }
        if (Object.keys(this._options).length > 0) {
            field.options = this._options;
        }
        if (this._validation) {
            field.validation = this._validation;
        }
        if (this._preview) {
            field.preview = this._preview;
        }
        if (this._fieldset) {
            field.fieldset = this._fieldset;
        }
        this.extendProperties(field);
        if (field.of) {
            field.of = subgenerateMany(field.of);
        }
        if (field.preview) {
            field.preview = subgenerate(field.preview);
        }
        return field;
    };
    return StandardGenerator;
}(BaseGenerator));
var GeneratorWithFields = /** @class */ (function (_super) {
    __extends(GeneratorWithFields, _super);
    function GeneratorWithFields(predefinedFields, type, name, title) {
        var _this = _super.call(this, type, name, title) || this;
        _this._fields = [];
        _this._predefinedFields = predefinedFields || {};
        return _this;
    }
    GeneratorWithFields.prototype.field = function (field) {
        // @TODO Better type checking here?
        if (typeof field === 'object') {
            this._fields.push(field);
        }
        else if (typeof field === 'string') {
            if (Object.prototype.hasOwnProperty.call(this._predefinedFields, field)) {
                this._fields.push(this._predefinedFields[field]);
            }
            else {
                throw new Error("Predefined field \"" + field + "\" not found.");
            }
        }
        return this;
    };
    GeneratorWithFields.prototype.fields = function (fields) {
        fields.forEach(this.field, this);
        return this;
    };
    GeneratorWithFields.prototype.extendProperties = function (field) {
        if (this._fields.length) {
            field.fields = this._fields;
        }
    };
    GeneratorWithFields.prototype.generate = function () {
        var field = _super.prototype.generate.call(this);
        if (field.fields) {
            field.fields = subgenerateMany(field.fields);
        }
        return field;
    };
    return GeneratorWithFields;
}(StandardGenerator));

var ArrayFieldGenerator = /** @class */ (function (_super) {
    __extends(ArrayFieldGenerator, _super);
    function ArrayFieldGenerator(predefinedFields, name, title) {
        var _this = _super.call(this, predefinedFields, 'array', name, title) || this;
        _this._options = {};
        return _this;
    }
    ArrayFieldGenerator.prototype.fields = function () {
        throw Error('Use the "of" method for arrays.');
    };
    ArrayFieldGenerator.prototype.of = function (fields) {
        return _super.prototype.fields.call(this, fields);
    };
    ArrayFieldGenerator.prototype.sortable = function (sortable) {
        if (sortable === void 0) { sortable = true; }
        this._options.sortable = sortable;
        return this;
    };
    ArrayFieldGenerator.prototype.layout = function (layout) {
        this._options.layout = layout;
        return this;
    };
    ArrayFieldGenerator.prototype.list = function (items) {
        this._options.list = items;
        return this;
    };
    ArrayFieldGenerator.prototype.editModal = function (editModal) {
        this._options.editModal = editModal;
        return this;
    };
    ArrayFieldGenerator.prototype.extendProperties = function (field) {
        if (this._fields.length)
            field.of = this._fields;
    };
    return ArrayFieldGenerator;
}(GeneratorWithFields));

var BlockFieldGenerator = /** @class */ (function (_super) {
    __extends(BlockFieldGenerator, _super);
    function BlockFieldGenerator(styles, lists, marks) {
        var _this = _super.call(this, 'block', '') || this;
        _this._styles = [];
        _this._lists = [];
        _this._marks = {};
        _this._styles = styles;
        _this._lists = lists;
        _this._marks = marks;
        return _this;
    }
    BlockFieldGenerator.prototype.extendProperties = function (field) {
        if (this._styles.length) {
            field.styles = this._styles;
        }
        if (this._lists.length) {
            field.lists = this._lists;
        }
        if (Object.keys(this._marks).length > 0) {
            field.marks = this._marks;
        }
    };
    BlockFieldGenerator.prototype.generate = function () {
        var _a;
        var field = _super.prototype.generate.call(this);
        if ((_a = field.marks) === null || _a === void 0 ? void 0 : _a.annotations) {
            field.marks.annotations = subgenerateMany(field.marks.annotations);
        }
        return field;
    };
    return BlockFieldGenerator;
}(StandardGenerator));
var BlocksFieldGenerator = /** @class */ (function (_super) {
    __extends(BlocksFieldGenerator, _super);
    function BlocksFieldGenerator(predefinedFields, name, title) {
        var _this = _super.call(this, predefinedFields, 'array', name, title) || this;
        _this._styles = [];
        _this._lists = [];
        _this._marks = {};
        return _this;
    }
    BlocksFieldGenerator.prototype.styles = function (styles) {
        this._styles = styles;
        return this;
    };
    BlocksFieldGenerator.prototype.lists = function (lists) {
        this._lists = lists;
        return this;
    };
    BlocksFieldGenerator.prototype.marks = function (marks) {
        this._marks = marks;
        return this;
    };
    BlocksFieldGenerator.prototype.fields = function () {
        throw Error('Use the "of" method for blocks.');
    };
    BlocksFieldGenerator.prototype.of = function (fields) {
        return _super.prototype.fields.call(this, fields);
    };
    BlocksFieldGenerator.prototype.extendProperties = function (field) {
        this._fields.unshift(new BlockFieldGenerator(this._styles, this._lists, this._marks));
        field.of = this._fields;
    };
    return BlocksFieldGenerator;
}(GeneratorWithFields));

var BooleanFieldGenerator = /** @class */ (function (_super) {
    __extends(BooleanFieldGenerator, _super);
    function BooleanFieldGenerator(name, title) {
        var _this = _super.call(this, 'boolean', name, title) || this;
        _this._options = {};
        return _this;
    }
    BooleanFieldGenerator.prototype.layout = function (layout) {
        this._options.layout = layout;
        return this;
    };
    return BooleanFieldGenerator;
}(StandardGenerator));

var DateFieldGenerator = /** @class */ (function (_super) {
    __extends(DateFieldGenerator, _super);
    function DateFieldGenerator(name, title) {
        var _this = _super.call(this, 'date', name, title) || this;
        _this._options = {};
        return _this;
    }
    DateFieldGenerator.prototype.format = function (format) {
        this._options.dateFormat = format;
        return this;
    };
    DateFieldGenerator.prototype.todayLabel = function (label) {
        this._options.calendarTodayLabel = label;
        return this;
    };
    return DateFieldGenerator;
}(StandardGenerator));

var DatetimeFieldGenerator = /** @class */ (function (_super) {
    __extends(DatetimeFieldGenerator, _super);
    function DatetimeFieldGenerator(name, title) {
        var _this = _super.call(this, 'datetime', name, title) || this;
        _this._options = {};
        return _this;
    }
    DatetimeFieldGenerator.prototype.format = function (_a) {
        var date = _a.date, time = _a.time;
        if (date) {
            this._options.dateFormat = date;
        }
        if (time) {
            this._options.timeFormat = time;
        }
        return this;
    };
    DatetimeFieldGenerator.prototype.step = function (step) {
        this._options.timeStep = step;
        return this;
    };
    DatetimeFieldGenerator.prototype.todayLabel = function (label) {
        this._options.calendarTodayLabel = label;
        return this;
    };
    return DatetimeFieldGenerator;
}(StandardGenerator));

var DocumentGenerator = /** @class */ (function (_super) {
    __extends(DocumentGenerator, _super);
    function DocumentGenerator(predefinedFields, name, title) {
        var _this = _super.call(this, predefinedFields, 'document', name, title) || this;
        _this._orderings = [];
        _this._fieldsets = [];
        return _this;
    }
    DocumentGenerator.prototype.liveEdit = function (liveEdit) {
        if (liveEdit === void 0) { liveEdit = true; }
        this._liveEdit = liveEdit;
        return this;
    };
    DocumentGenerator.prototype.icon = function (icon) {
        this._icon = icon;
        return this;
    };
    DocumentGenerator.prototype.orderings = function (orderings) {
        this._orderings = orderings;
        return this;
    };
    DocumentGenerator.prototype.fieldsets = function (sets) {
        this._fieldsets = sets;
        return this;
    };
    DocumentGenerator.prototype.extendProperties = function (field) {
        _super.prototype.extendProperties.call(this, field);
        if (this._orderings.length) {
            field.orderings = this._orderings;
        }
        if (this._icon) {
            field.icon = this._icon;
        }
        if (this._liveEdit) {
            field.liveEdit = true;
        }
        if (this._fieldsets.length) {
            field.fieldsets = this._fieldsets;
        }
    };
    DocumentGenerator.prototype.generate = function () {
        var field = _super.prototype.generate.call(this);
        if (field.orderings) {
            field.orderings = subgenerateMany(field.orderings);
        }
        if (field.fieldsets) {
            field.fieldsets = subgenerateMany(field.fieldsets);
        }
        return field;
    };
    return DocumentGenerator;
}(GeneratorWithFields));

var FieldGenerator = /** @class */ (function (_super) {
    __extends(FieldGenerator, _super);
    function FieldGenerator(predefinedFields, type, name, title) {
        return _super.call(this, predefinedFields, type, name || type, title) || this;
    }
    return FieldGenerator;
}(GeneratorWithFields));

var FieldSetGenerator = /** @class */ (function (_super) {
    __extends(FieldSetGenerator, _super);
    function FieldSetGenerator(name, title) {
        var _this = _super.call(this, name, title) || this;
        _this._options = {};
        _this._name = name;
        return _this;
    }
    FieldSetGenerator.prototype.collapsible = function (isCollapsible) {
        if (isCollapsible === void 0) { isCollapsible = true; }
        this._options.collapsible = isCollapsible;
        return this;
    };
    FieldSetGenerator.prototype.collapsed = function (isCollapsed) {
        if (isCollapsed === void 0) { isCollapsed = true; }
        this._options.collapsed = isCollapsed;
        return this;
    };
    FieldSetGenerator.prototype.columns = function (count) {
        this._options.columns = count;
        return this;
    };
    FieldSetGenerator.prototype.generate = function () {
        var fieldset = {
            name: this._name,
        };
        fieldset.title = generateTitle(this._name, this._title);
        if (Object.keys(this._options).length > 0) {
            fieldset.options = this._options;
        }
        return fieldset;
    };
    return FieldSetGenerator;
}(BaseGenerator));

var FileFieldGenerator = /** @class */ (function (_super) {
    __extends(FileFieldGenerator, _super);
    function FileFieldGenerator(predefinedFields, name, title) {
        var _this = _super.call(this, predefinedFields, 'file', name, title) || this;
        _this._options = {};
        return _this;
    }
    FileFieldGenerator.prototype.accept = function (accept) {
        this._options.accept = accept;
        return this;
    };
    FileFieldGenerator.prototype.storeOriginalFilename = function (storeOriginalFilename) {
        if (storeOriginalFilename === void 0) { storeOriginalFilename = true; }
        this._options.storeOriginalFilename = storeOriginalFilename;
        return this;
    };
    return FileFieldGenerator;
}(GeneratorWithFields));

var GeopointFieldGenerator = /** @class */ (function (_super) {
    __extends(GeopointFieldGenerator, _super);
    function GeopointFieldGenerator(name, title) {
        return _super.call(this, 'geopoint', name, title) || this;
    }
    return GeopointFieldGenerator;
}(StandardGenerator));

var alt = {
    name: 'alt',
    title: 'Alt Text',
    type: 'string',
};
var caption = {
    name: 'caption',
    title: 'Caption',
    type: 'array',
    of: [{ type: 'block' }],
};
var ImageFieldGenerator = /** @class */ (function (_super) {
    __extends(ImageFieldGenerator, _super);
    function ImageFieldGenerator(predefinedFields, name, title) {
        var _this = _super.call(this, predefinedFields, 'image', name, title) || this;
        _this._options = {};
        _this._predefinedFields = {
            alt: alt,
            caption: caption,
        };
        return _this;
    }
    ImageFieldGenerator.prototype.metadata = function (metadata) {
        this._options.metadata = metadata;
        return this;
    };
    ImageFieldGenerator.prototype.hotspot = function (hotspot) {
        if (hotspot === void 0) { hotspot = true; }
        this._options.hotspot = hotspot;
        return this;
    };
    ImageFieldGenerator.prototype.storeOriginalFilename = function (store) {
        if (store === void 0) { store = true; }
        this._options.storeOriginalFilename = store;
        return this;
    };
    ImageFieldGenerator.prototype.accept = function (mimeType) {
        this._options.accept = mimeType;
        return this;
    };
    ImageFieldGenerator.prototype.sources = function (sources) {
        this._options.sources = sources;
        return this;
    };
    return ImageFieldGenerator;
}(GeneratorWithFields));

var NumberFieldGenerator = /** @class */ (function (_super) {
    __extends(NumberFieldGenerator, _super);
    function NumberFieldGenerator(name, title) {
        var _this = _super.call(this, 'number', name, title) || this;
        _this._options = {};
        return _this;
    }
    NumberFieldGenerator.prototype.list = function (items) {
        this._options.list = items;
        return this;
    };
    return NumberFieldGenerator;
}(StandardGenerator));

var ObjectFieldGenerator = /** @class */ (function (_super) {
    __extends(ObjectFieldGenerator, _super);
    function ObjectFieldGenerator(predefinedFields, name, title) {
        var _this = _super.call(this, predefinedFields, 'object', name, title) || this;
        _this._fieldsets = [];
        _this._options = {};
        return _this;
    }
    ObjectFieldGenerator.prototype.collapsible = function (isCollapsible) {
        if (isCollapsible === void 0) { isCollapsible = true; }
        this._options.collapsible = isCollapsible;
        return this;
    };
    ObjectFieldGenerator.prototype.collapsed = function (isCollapsed) {
        if (isCollapsed === void 0) { isCollapsed = true; }
        this._options.collapsed = isCollapsed;
        return this;
    };
    ObjectFieldGenerator.prototype.fieldsets = function (sets) {
        this._fieldsets = sets;
        return this;
    };
    ObjectFieldGenerator.prototype.inputComponent = function (component) {
        this._inputComponent = component;
        return this;
    };
    ObjectFieldGenerator.prototype.extendProperties = function (field) {
        _super.prototype.extendProperties.call(this, field);
        if (this._fieldsets.length) {
            field.fieldsets = this._fieldsets;
        }
        if (this._inputComponent) {
            field.inputComponent = this._inputComponent;
        }
    };
    ObjectFieldGenerator.prototype.generate = function () {
        var field = _super.prototype.generate.call(this);
        if (field.fieldsets) {
            field.fieldsets = subgenerateMany(field.fieldsets);
        }
        return field;
    };
    return ObjectFieldGenerator;
}(GeneratorWithFields));

var ReferenceFieldGenerator = /** @class */ (function (_super) {
    __extends(ReferenceFieldGenerator, _super);
    function ReferenceFieldGenerator(name, title) {
        var _this = _super.call(this, 'reference', name, title) || this;
        _this._references = [];
        _this._filter = '';
        _this._filterParams = undefined;
        return _this;
    }
    ReferenceFieldGenerator.prototype.add = function (reference) {
        if (typeof reference === 'string') {
            this._references.push({ type: reference });
        }
        else if (typeof reference === 'object') {
            this._references.push(reference);
        }
        return this;
    };
    ReferenceFieldGenerator.prototype.filter = function (filter, params) {
        // groq string or function
        this._filter = filter;
        this._filterParams = params;
        return this;
    };
    ReferenceFieldGenerator.prototype.to = function (references) {
        this._references = references;
        return this;
    };
    ReferenceFieldGenerator.prototype.weak = function (isWeak) {
        if (isWeak === void 0) { isWeak = true; }
        this._weak = isWeak;
        return this;
    };
    ReferenceFieldGenerator.prototype.extendProperties = function (field) {
        if (this._weak !== undefined) {
            field.weak = this._weak;
        }
        if (this._references.length) {
            field.to = this._references;
        }
        if (this._filter) {
            field.options = {
                filter: this._filter,
            };
            if (this._filterParams) {
                field.options.filterParams = this._filterParams;
            }
        }
    };
    return ReferenceFieldGenerator;
}(StandardGenerator));

var SlugFieldGenerator = /** @class */ (function (_super) {
    __extends(SlugFieldGenerator, _super);
    function SlugFieldGenerator(name, title) {
        var _this = _super.call(this, 'slug', name, title) || this;
        _this._options = {};
        return _this;
    }
    SlugFieldGenerator.prototype.source = function (source) {
        this._options.source = source;
        return this;
    };
    SlugFieldGenerator.prototype.maxLength = function (maxLength) {
        this._options.maxLength = maxLength;
        return this;
    };
    SlugFieldGenerator.prototype.slugify = function (slugifyFn) {
        this._options.slugify = slugifyFn;
        return this;
    };
    SlugFieldGenerator.prototype.isUnique = function (fn) {
        this._options.isUnique = fn;
        return this;
    };
    return SlugFieldGenerator;
}(StandardGenerator));

var StringFieldGenerator = /** @class */ (function (_super) {
    __extends(StringFieldGenerator, _super);
    function StringFieldGenerator(name, title) {
        var _this = _super.call(this, 'string', name, title) || this;
        _this._options = {};
        return _this;
    }
    StringFieldGenerator.prototype.list = function (items) {
        this._options.list = items;
        return this;
    };
    StringFieldGenerator.prototype.layout = function (layout) {
        this._options.layout = layout;
        return this;
    };
    StringFieldGenerator.prototype.direction = function (direction) {
        this._options.direction = direction;
        return this;
    };
    return StringFieldGenerator;
}(StandardGenerator));

var TextFieldGenerator = /** @class */ (function (_super) {
    __extends(TextFieldGenerator, _super);
    function TextFieldGenerator(name, title) {
        return _super.call(this, 'text', name, title) || this;
    }
    TextFieldGenerator.prototype.rows = function (count) {
        this._rows = count;
        return this;
    };
    TextFieldGenerator.prototype.extendProperties = function (field) {
        if (this._rows) {
            field.rows = this._rows;
        }
    };
    return TextFieldGenerator;
}(StandardGenerator));

var UrlFieldGenerator = /** @class */ (function (_super) {
    __extends(UrlFieldGenerator, _super);
    function UrlFieldGenerator(name, title) {
        return _super.call(this, 'url', name, title) || this;
    }
    return UrlFieldGenerator;
}(StandardGenerator));

var OrderingGenerator = /** @class */ (function (_super) {
    __extends(OrderingGenerator, _super);
    function OrderingGenerator(name, title) {
        var _this = _super.call(this, name, title) || this;
        _this._orders = [];
        return _this;
    }
    OrderingGenerator.prototype.by = function (field, direction) {
        this._orders.push({ field: field, direction: direction });
        return this;
    };
    OrderingGenerator.prototype.generate = function () {
        if (!this._name)
            throw Error('Name is required');
        var ordering = {
            name: this._name,
        };
        if (this._title)
            ordering.title = this._title;
        if (this._orders.length)
            ordering.by = this._orders;
        return ordering;
    };
    return OrderingGenerator;
}(BaseGenerator));

var PreviewGenerator = /** @class */ (function () {
    function PreviewGenerator(select) {
        if (select === void 0) { select = {}; }
        this._select = select;
    }
    PreviewGenerator.prototype.select = function (property, selection) {
        selection = selection || property;
        this._select[property] = selection;
        return this;
    };
    PreviewGenerator.prototype.prepare = function (fn) {
        this._prepare = fn;
        return this;
    };
    PreviewGenerator.prototype.component = function (component) {
        this._component = component;
        return this;
    };
    PreviewGenerator.prototype.generate = function () {
        if (Object.keys(this._select).length > 0) {
            var preview = {
                select: this._select,
            };
            if (this._prepare)
                preview.prepare = this._prepare;
            if (this._component)
                preview.component = this._component;
            return preview;
        }
    };
    return PreviewGenerator;
}());

var SchemaBuilder = /** @class */ (function () {
    function SchemaBuilder(predefinedFields) {
        // Base
        this.f = this.field;
        // Field types
        this.arr = this.array;
        this.bool = this.boolean;
        this.dt = this.datetime;
        this.doc = this.document;
        this.fset = this.fieldset;
        this.geo = this.geopoint;
        this.img = this.image;
        this.num = this.number;
        this.obj = this.object;
        this.ref = this.reference;
        this.str = this.string;
        // Orderings
        this.sort = this.ordering;
        // Preview
        this.view = this.ordering;
        this._predefinedFields = predefinedFields || {};
    }
    SchemaBuilder.prototype.define = function (key, field) {
        this._predefinedFields[key] = field;
    };
    SchemaBuilder.prototype.field = function (type, name, title) {
        return new FieldGenerator(this._predefinedFields, type, name, title);
    };
    SchemaBuilder.prototype.array = function (name, title) {
        return new ArrayFieldGenerator(this._predefinedFields, name, title);
    };
    SchemaBuilder.prototype.blocks = function (name, title) {
        return new BlocksFieldGenerator(this._predefinedFields, name, title);
    };
    SchemaBuilder.prototype.boolean = function (name, title) {
        return new BooleanFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.date = function (name, title) {
        return new DateFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.datetime = function (name, title) {
        return new DatetimeFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.document = function (name, title) {
        return new DocumentGenerator(this._predefinedFields, name, title);
    };
    SchemaBuilder.prototype.fieldset = function (name, title) {
        return new FieldSetGenerator(name, title);
    };
    SchemaBuilder.prototype.file = function (name, title) {
        return new FileFieldGenerator(this._predefinedFields, name, title);
    };
    SchemaBuilder.prototype.geopoint = function (name, title) {
        return new GeopointFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.image = function (name, title) {
        return new ImageFieldGenerator(this._predefinedFields, name, title);
    };
    SchemaBuilder.prototype.number = function (name, title) {
        return new NumberFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.object = function (name, title) {
        return new ObjectFieldGenerator(this._predefinedFields, name, title);
    };
    SchemaBuilder.prototype.reference = function (name, title) {
        return new ReferenceFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.slug = function (name, title) {
        return new SlugFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.string = function (name, title) {
        return new StringFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.text = function (name, title) {
        return new TextFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.url = function (name, title) {
        return new UrlFieldGenerator(name, title);
    };
    SchemaBuilder.prototype.ordering = function (name, title) {
        return new OrderingGenerator(name, title);
    };
    SchemaBuilder.prototype.preview = function (select) {
        return new PreviewGenerator(select);
    };
    return SchemaBuilder;
}());

exports.SchemaBuilder = SchemaBuilder;
